@@ -71,7 +71,7 @@ int write_img_inventory(void)
 
 	crt.state = TASK_ALIVE;
 	crt.pid.real = getpid();
-	if (get_task_ids(&crt)){
+	if (get_task_ids(&crt)) {
 		close(fd);
 		return -1;
 	}
@@ -240,30 +240,18 @@ int open_image_dir(char *dir)
 	}
 
 	ret = install_service_fd(IMG_FD_OFF, fd);
+	close(fd);
+	fd = ret;
 
 	if (opts.img_parent) {
-		int pfd;
-
 		ret = symlinkat(opts.img_parent, fd, CR_PARENT_LINK);
-		if (ret < 0) {
-			pr_perror("Can't link parent snapshot.");
+		if (ret < 0 && errno != EEXIST) {
+			pr_perror("Can't link parent snapshot");
 			goto err;
 		}
-
-		pfd = openat(fd, CR_PARENT_LINK, O_RDONLY);
-		if (pfd < 0) {
-			pr_perror("Can't open parent snapshot.");
-			goto err;
-		}
-
-		ret = install_service_fd(PARENT_FD_OFF, pfd);
-
-		close(pfd);
 	}
 
-	close(fd);
-
-	return ret;
+	return 0;
 
 err:
 	close_image_dir();
@@ -295,7 +283,7 @@ int open_pages_image_at(int dfd, unsigned long flags, int pm_fd)
 {
 	unsigned id;
 
-	if (flags == O_RDONLY) {
+	if (flags == O_RDONLY || flags == O_RDWR) {
 		PagemapHead *h;
 		if (pb_read_one(pm_fd, &h, PB_PAGEMAP_HEAD) < 0)
 			return -1;
