@@ -10,4 +10,8 @@
 extern size_t strlcpy(char *dest, const char *src, size_t size);
 #endif
 
+#ifndef CONFIG_HAS_STRLCAT
+extern size_t strlcat(char *dest, const char *src, size_t count);
+#endif
+
 #endif /* __CR_STRING_H__ */
