@@ -73,6 +73,7 @@ static/fifo_wronly
 static/zombie00
 static/rlimits00
 transition/fork
+transition/thread-bomb
 static/pty00
 static/pty01
 static/pty04
@@ -97,6 +98,7 @@ static/grow_map02
 static/stopped
 static/chroot
 static/chroot-file
+static/rtc
 "
 # Duplicate list with ns/ prefix
 TEST_LIST=$TEST_LIST$(echo $TEST_LIST | tr ' ' '\n' | sed 's#^#ns/#')
@@ -147,6 +149,7 @@ sk-netlink
 tun
 chroot
 chroot-file
+rtc
 "
 
 source $(readlink -f `dirname $0`/env.sh) || exit 1
@@ -172,7 +175,8 @@ BATCH_TEST=0
 SPECIFIED_NAME_USED=0
 
 zdtm_sep()
-{
+{ (
+	set +x
 	local msg=$1
 	[ -n "$msg" ] && msg=" $msg "
 	awk -v m=${2:-=} -v "msg=$msg" '
@@ -183,7 +187,7 @@ zdtm_sep()
 			gsub(/ /, m, sep);
 			printf("%s%s%s\n",sep,msg,sep);
 		}' < /dev/null
-}
+) }
 
 check_criu()
 {
@@ -354,7 +358,7 @@ stop_test()
 save_fds()
 {
 	test -n "$PIDNS" && return 0
-	ls -l /proc/$1/fd | sed 's/\(-> \(pipe\|socket\)\):.*/\1/' | awk '{ print $9,$10,$11; }' > $2
+	ls -l /proc/$1/fd | sed 's/\(-> \(pipe\|socket\)\):.*/\1/' | sed -e 's/\/.nfs[0-9a-zA-Z]*/.nfs-silly-rename/' | awk '{ print $9,$10,$11; }' > $2
 }
 
 save_maps()
@@ -398,6 +402,9 @@ run_test()
 	if [[ $1 =~ "unlink_" ]]; then
 		linkremap="--link-remap"
 	fi
+	if [[ $1 =~ "write_read10" ]]; then
+		linkremap="--link-remap"
+	fi
 
 	if [ -n "$MAINSTREAM_KERNEL" ] && [ $COMPILE_ONLY -eq 0 ] && echo $TEST_CR_KERNEL | grep -q ${test#ns/}; then
 		echo "Skip $test"
@@ -441,9 +448,14 @@ EOF
 		args="--root $ZDTM_ROOT --pidfile $TPID $args"
 	fi
 
+	if [ $tname = "rtc" ]; then
+		args="$args -L `pwd`/$tdir/lib"
+	fi
+
 	for i in `seq $ITERATIONS`; do
 		local dump_only=
 		local postdump=
+		local dump_cmd="dump"
 		ddump=`readlink -fm dump/$tname/$PID/$i`
 		DUMP_PATH=$ddump
 		echo Dump $PID
@@ -466,6 +478,7 @@ EOF
 			if [ "$i" -ne "$ITERATIONS" ]; then
 				snapopt="$snapopt -R --track-mem"
 				dump_only=1
+				[ -n "$PRE_DUMP" ] && dump_cmd="pre-dump"
 			fi
 			[ -n "$snappdir" ] && snapopt="$snapopt --prev-images-dir=$snappdir"
 		fi
@@ -474,7 +487,7 @@ EOF
 
 		save_fds $PID  $ddump/dump.fd
 		save_maps $PID  $ddump/dump.maps
-		setsid $CRIU_CPT dump $opts --file-locks --tcp-established $linkremap \
+		setsid $CRIU_CPT $dump_cmd $opts --file-locks --tcp-established $linkremap \
 			-x --evasive-devices -D $ddump -o dump.log -v4 -t $PID $args $ARGS $snapopt $postdump
 		retcode=$?
 
@@ -509,7 +522,9 @@ EOF
 			diff_fds $ddump/dump.fd $ddump/dump.fd.after || return 1
 
 			save_maps $PID  $ddump/dump.maps.after
-			diff_maps $ddump/dump.maps $ddump/dump.maps.after || return 1
+			expr $tname : "static" > /dev/null && {
+				diff_maps $ddump/dump.maps $ddump/dump.maps.after || return 1
+			}
 
 			if [[ $linkremap ]]; then
 				echo "remove ./$tdir/link_remap.*"
@@ -542,7 +557,9 @@ EOF
 			[ $i -eq 5 ] && return 2
 
 			save_maps $PID $ddump/restore.maps
-			diff_maps $ddump/dump.maps $ddump/restore.maps || return 2
+			expr $tname : "static" > /dev/null && {
+				diff_maps $ddump/dump.maps $ddump/restore.maps || return 2
+			}
 		fi
 
 	done
@@ -653,7 +670,9 @@ Options:
 	-g : Generate executables only
 	-n : Batch test
 	-r : Run test with specified name directly without match or check
-	-v : Verbose mode 
+	-v : Verbose mode
+	-P : Make pre-dump instead of dump on all iterations except the last one
+	-s : Make iterative snapshots. Only the last one will be checked.
 EOF
 }
 
@@ -693,7 +712,11 @@ while :; do
 		;;
 	  -x)
 		shift
-		EXCLUDE_PATTERN=$1
+		if [ -z "$EXCLUDE_PATTERN" ]; then
+			EXCLUDE_PATTERN=$1
+		else
+			EXCLUDE_PATTERN="${EXCLUDE_PATTERN}\|$1"
+		fi
 		shift
 		;;
 	  -t)
@@ -708,6 +731,19 @@ while :; do
 		shift
 		;;
 	  -s)
+		if [ -n "$PRE_DUMP" ]; then
+			echo "-P and -s can not be used together"
+			exit 1
+		fi
+		SNAPSHOT=1
+		shift
+		;;
+	  -P)
+		if [ -n "$SNAPSHOT" ]; then
+			echo "-P and -s can not be used together"
+			exit 1
+		fi
+		PRE_DUMP=1
 		SNAPSHOT=1
 		shift
 		;;
